@@ -5,14 +5,14 @@ if (!library(Rmpi,logical.return = TRUE)){
 options(error=quote(assign(".mpi.err", TRUE, env = .GlobalEnv)))
 .comm <- 1
 .intercomm <- 2
-mpi.comm.get.parent(.intercomm)
-mpi.intercomm.merge(.intercomm,1,.comm)
-mpi.comm.set.errhandler(.comm)
-mpi.comm.disconnect(.intercomm)
+invisible(mpi.comm.get.parent(.intercomm))
+invisible(mpi.intercomm.merge(.intercomm,1,.comm))
+invisible(mpi.comm.set.errhandler(.comm))
+invisible(mpi.comm.disconnect(.intercomm))
 mpi.hostinfo(.comm)
 repeat 
     try(eval(mpi.bcast.cmd(rank=0,comm=.comm),envir=sys.parent()),TRUE)
 print("Done")
-mpi.comm.disconnect(.comm)
-#mpi.abort(0)
+invisible(mpi.comm.disconnect(.comm))
+invisible(mpi.comm.set.errhandler(0))
 mpi.quit()
